@@ -7,6 +7,8 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
+
 import io
 import os
 import random
@@ -14,19 +16,37 @@ import re
 import sys
 import time
 import traceback
+from test.helpers import type_validator_factory
+from typing import List
 
 import pytest
 
-from b2sdk.b2http import B2Http
-from b2sdk.encryption.setting import EncryptionAlgorithm, EncryptionMode, EncryptionSetting
-from b2sdk.replication.setting import ReplicationConfiguration, ReplicationRule
-from b2sdk.replication.types import ReplicationStatus
-from b2sdk.file_lock import BucketRetentionSetting, NO_RETENTION_FILE_SETTING, RetentionMode, RetentionPeriod
-from b2sdk.raw_api import ALL_CAPABILITIES, B2RawHTTPApi, REALM_URLS
-from b2sdk.utils import hex_sha1_of_stream
+from b2sdk._internal.b2http import B2Http
+from b2sdk._internal.encryption.setting import (
+    EncryptionAlgorithm,
+    EncryptionMode,
+    EncryptionSetting,
+)
+from b2sdk._internal.exception import DisablingFileLockNotSupported, Unauthorized
+from b2sdk._internal.file_lock import (
+    NO_RETENTION_FILE_SETTING,
+    BucketRetentionSetting,
+    FileRetentionSetting,
+    RetentionMode,
+    RetentionPeriod,
+)
+from b2sdk._internal.raw_api import (
+    ALL_CAPABILITIES,
+    REALM_URLS,
+    B2RawHTTPApi,
+    NotificationRuleResponse,
+)
+from b2sdk._internal.replication.setting import ReplicationConfiguration, ReplicationRule
+from b2sdk._internal.replication.types import ReplicationStatus
+from b2sdk._internal.utils import hex_sha1_of_stream
 
 
-# TODO: rewrite to separate test cases
+# TODO: rewrite to separate test cases after introduction of reusable bucket
 def test_raw_api(dont_cleanup_old_buckets):
     """
     Exercise the code in B2RawHTTPApi by making each call once, just
@@ -90,9 +110,16 @@ def raw_api_test_helper(raw_api, should_cleanup_old_buckets):
     # b2_authorize_account
     print('b2_authorize_account')
     auth_dict = authorize_raw_api(raw_api)
-    missing_capabilities = set(ALL_CAPABILITIES) - {'readBuckets', 'listAllBucketNames'
-                                                   } - set(auth_dict['allowed']['capabilities'])
-    assert not missing_capabilities, 'it appears that the raw_api integration test is being run with a non-full key. Missing capabilities: %s' % (
+
+    preview_feature_caps = {
+        'readBucketNotifications',
+        'writeBucketNotifications',
+    }
+    missing_capabilities = (
+        set(ALL_CAPABILITIES) - {'readBuckets', 'listAllBucketNames'} - preview_feature_caps -
+        set(auth_dict['allowed']['capabilities'])
+    )
+    assert not missing_capabilities, 'it appears that the raw_api integration test is being run with a non-full key. Missing capabilities: {}'.format(
         missing_capabilities,
     )
 
@@ -356,9 +383,17 @@ def raw_api_test_helper(raw_api, should_cleanup_old_buckets):
         len(file_contents),
         'text/plain',
         file_sha1,
-        {'color': 'blue'},
+        {
+            'color': 'blue',
+            'b2-cache-control': 'private, max-age=2222'
+        },
         io.BytesIO(file_contents),
         server_side_encryption=sse_b2_aes,
+        #custom_upload_timestamp=12345,
+        file_retention=FileRetentionSetting(
+            RetentionMode.GOVERNANCE,
+            int(time.time() + 100) * 1000,
+        )
     )
 
     file_id = file_dict['fileId']
@@ -367,6 +402,9 @@ def raw_api_test_helper(raw_api, should_cleanup_old_buckets):
     print('b2_list_file_versions')
     list_versions_dict = raw_api.list_file_versions(api_url, account_auth_token, bucket_id)
     assert [file_name] == [f_dict['fileName'] for f_dict in list_versions_dict['files']]
+    assert ['private, max-age=2222'] == [
+        f_dict['fileInfo']['b2-cache-control'] for f_dict in list_versions_dict['files']
+    ]
 
     # b2_download_file_by_id with auth
     print('b2_download_file_by_id (auth)')
@@ -519,9 +557,39 @@ def raw_api_test_helper(raw_api, should_cleanup_old_buckets):
         default_retention=BucketRetentionSetting(
             mode=RetentionMode.GOVERNANCE, period=RetentionPeriod(days=1)
         ),
+        is_file_lock_enabled=True,
     )
     assert first_bucket_revision < updated_bucket['revision']
 
+    # NOTE: this update_bucket call is only here to be able to find out the error code returned by
+    # the server if an attempt is made to disable file lock.  It has to be done here since the CLI
+    # by design does not allow disabling file lock at all (i.e. there is no --fileLockEnabled=false
+    # option or anything equivalent to that).
+    with pytest.raises(DisablingFileLockNotSupported):
+        raw_api.update_bucket(
+            api_url,
+            account_auth_token,
+            account_id,
+            bucket_id,
+            'allPrivate',
+            is_file_lock_enabled=False,
+        )
+
+    # b2_delete_file_version
+    print('b2_delete_file_version')
+
+    with pytest.raises(Unauthorized):
+        raw_api.delete_file_version(api_url, account_auth_token, file_id, file_name)
+    raw_api.delete_file_version(api_url, account_auth_token, file_id, file_name, True)
+
+    print('b2_get_bucket_notification_rules & b2_set_bucket_notification_rules')
+    try:
+        _subtest_bucket_notification_rules(
+            raw_api, auth_dict, api_url, account_auth_token, bucket_id
+        )
+    except pytest.skip.Exception as e:
+        print(e)
+
     # Clean up this test.
     _clean_and_delete_bucket(raw_api, api_url, account_auth_token, account_id, bucket_id)
 
@@ -530,6 +598,48 @@ def raw_api_test_helper(raw_api, should_cleanup_old_buckets):
         _cleanup_old_buckets(raw_api, auth_dict, bucket_list_dict)
 
 
+def _subtest_bucket_notification_rules(raw_api, auth_dict, api_url, account_auth_token, bucket_id):
+    if 'writeBucketNotifications' not in auth_dict['allowed']['capabilities']:
+        pytest.skip('Test account does not have writeBucketNotifications capability')
+
+    notification_rule = {
+        'eventTypes': ['b2:ObjectCreated:Copy'],
+        'isEnabled': False,
+        'name': 'test-notification-rule',
+        'objectNamePrefix': 'test/object/prefix/',
+        'targetConfiguration':
+            {
+                'targetType': 'webhook',
+                'url': 'https://example.com/webhook',
+                'hmacSha256SigningSecret': 'a' * 32,
+            },
+    }
+
+    notification_rules_response_list = raw_api.set_bucket_notification_rules(
+        api_url, account_auth_token, bucket_id, [notification_rule]
+    )
+    notification_rule_response_list_validate = type_validator_factory(
+        List[NotificationRuleResponse]
+    )
+    notification_rule_response_list_validate(notification_rules_response_list)
+    expected_notification_rule_response_list = [
+        {
+            **notification_rule, 'isSuspended': False,
+            'suspensionReason': '',
+            'targetConfiguration':
+                {
+                    **notification_rule['targetConfiguration'],
+                    'customHeaders': None,
+                    'hmacSha256SigningSecret': 'a' * 32,
+                }
+        }
+    ]
+    assert notification_rules_response_list == expected_notification_rule_response_list
+
+    assert raw_api.set_bucket_notification_rules(api_url, account_auth_token, bucket_id, []) == []
+    assert raw_api.get_bucket_notification_rules(api_url, account_auth_token, bucket_id) == []
+
+
 def cleanup_old_buckets():
     raw_api = B2RawHTTPApi(B2Http())
     auth_dict = authorize_raw_api(raw_api)
